import React, { Dispatch } from 'react';
import { MuiThemeProvider as ThemeProvider, StylesProvider } from "@material-ui/core/styles";
import { studentTheme } from "./ui/theme";
import { MuiPickersUtilsProvider } from "@material-ui/pickers";
import MomentUtils from "@date-io/moment";
import { BrowserRouter, Link, Route, Switch } from "react-router-dom"

import "moment/locale/pl"
import '@/styles/overrides.scss'
import '@/styles/header.scss'

import moment, { Moment } from "moment";
import { route, routes } from "@/routing";
import { Provider, useDispatch, useSelector } from "react-redux";
import store from "@/state/store";
import { AppState } from "@/state/reducer";
import { StudentAction, StudentActions } from "@/state/actions/student";
import { sampleStudent } from "@/provider/dummy/student";

moment.locale("pl")

class LocalizedMomentUtils extends MomentUtils {
    getDatePickerHeaderText(date: Moment): string {
        return this.format(date, "d MMM yyyy");
    }
}

const UserMenu = () => {
    const student = useSelector<AppState>(state => state.student);

    const dispatch = useDispatch<Dispatch<StudentAction>>();

    const handleUserLogin = () => {
        dispatch({
            type: StudentActions.Login,
            student: sampleStudent,
        })
    }

    const handleUserLogout = () => {
        dispatch({ type: StudentActions.Logout })
    }

    return student ? <>
        zalogowany jako: <strong>Jan Kowalski</strong>
        {' '}
        (<Link to={'#'} onClick={ handleUserLogout }>wyloguj się</Link>)
    </> : <>
        <Link to={'#'} onClick={ handleUserLogin }>zaloguj się</Link>
    </>;
}

const AppHeader = () => {
    return <header className="header">
        <div id="logo" className="header__logo">
            <Link to={ route('home') }>
                <img src="img/pg-logotyp.svg"/>
            </Link>
        </div>
        <div className="header__nav">
            <nav className="header__top">
                <ul className="header__menu"></ul>
                <div className="header__user">
                    <UserMenu />
                </div>
                <div className="header__divider" />
                <ul className="header__language-switcher">
                    <li>pl</li>
                    <li>en</li>
                </ul>
            </nav>
            <nav className="header__bottom">
                <ul className="header__menu header__menu--main"></ul>
            </nav>
        </div>
    </header>
}


function App() {
    return (
        <Provider store={ store }>
            <StylesProvider injectFirst>
                <MuiPickersUtilsProvider utils={ LocalizedMomentUtils } libInstance={ moment }>
                    <ThemeProvider theme={ studentTheme }>
                        <BrowserRouter>
                            <AppHeader />
                            <Switch>{ routes.map(({ name, content, ...route }) => <Route { ...route } key={ name }>{ content() }</Route>) }</Switch>
                        </BrowserRouter>
                    </ThemeProvider>
                </MuiPickersUtilsProvider>
            </StylesProvider>
        </Provider>
    );
}

export default App;
